/*
	Helper to interactivly establish an smbmount. This
	command is generally used from a persmount.conf file
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/ioctl.h>
#include <translat.h>
#include <misc.h>
#include <popen.h>
#include <confdb.h>
#include <dialog.h>

static CONFIG_FILE f_volinfo (".virtualfs/share.conf",help_nil
	,CONFIGF_MANAGED|CONFIGF_OPTIONAL
	,NULL,NULL,0600);

static void usage()
{
	fprintf (stderr,"virtualfs-xsmb --server server --share share --mountdir mount_point\n");
	fprintf (stderr,"	[ --account account ]\n");
}


static void loadmsg()
{
	linuxconf_loadlibmsg ("/usr/lib/virtualfs");
}


int main (int argc, char *argv[])
{
	int ret=-1;
	loadmsg();
	SSTRING server;
	SSTRING account;
	SSTRING share;
	SSTRING mountdir;
	for (int i=1; i<argc; i++){
		const char *arg = argv[i];
		const char *nextarg = argv[i+1];
		if (strcmp(arg,"--server")==0){
			server.setfrom (nextarg);
			i++;
		}else if (strcmp(arg,"--share")==0){
			share.setfrom (nextarg);
			i++;
		}else if (strcmp(arg,"--account")==0){
			account.setfrom (nextarg);
			i++;
		}else if (strcmp(arg,"--mountdir")==0){
			mountdir.setfrom (nextarg);
			i++;
		}else if (strcmp(arg,"--map")==0){
			if (nextarg[0] == '/') nextarg++;
			char *serv = strchr(nextarg,'/');
			if (serv != NULL){
				serv++;
				serv = strchr(serv,'/');
				if (serv != NULL){
					serv++;
					char *sha = strchr(serv,'/');
					if (sha != NULL){
						*sha++ = '\0';
						server.setfrom (serv);
						share.setfrom (sha);
					}
				}
			}
			i++;
		}else{
			usage();
		}
	}
	if (server.is_empty()){
		fprintf (stderr,"missing server\n");
		usage();
	}else if (share.is_empty()){
		fprintf (stderr,"missing share\n");
		usage();
	}else if (mountdir.is_empty()){
		fprintf (stderr,"missing mount point\n");
		usage();
	}else{
		CONFDB conf (f_volinfo);
		SSTRING accopt;
		SSTRING key;
		key.setfromf ("%s/%s",server.get(),share.get());
		if (account.is_empty()){
			struct passwd *p = getpwuid(getuid());
			const char *name = "";
			if (p != NULL) name = p->pw_name;
			account.setfrom (conf.getval("connectas",key.get(),name));
		}
		SSTRING passwd;
		passwd.setfrom (conf.getval("passwd",key.get(),""));

		if (passwd.is_empty()){
			dialog_clear();
			DIALOG dia;
			dia.settype (DIATYPE_POPUP);
			dia.newf_str ("Connect as",account);
			dia.newf_pass ("Password",passwd);
			char save = 0;
			dia.newf_chk ("",save,"Preserve password for next session");
			int nof = 0;
			SSTRING title;
			title.setfromf ("Connecting to share //%s/%s",server.get()
				,share.get());
			if (dia.edit (title.get(),"",help_nil,nof)!=MENU_ACCEPT){
				exit (-1);
			}else if (save){
				conf.replace ("passwd",key.get(),passwd);
				conf.replace ("connectas",key.get(),account);
				conf.save();
			}
		}
		if (!account.is_empty()){
			accopt.setfromf ("-U %s",account.get());
		}
		SSTRING cmd;
		cmd.setfromf ("/usr/bin/smbmount //%s/%s -c \"mount %s\" %s"
			,server.get(),share.get(),mountdir.get(),accopt.get());
		int fd = open ("/dev/tty",O_RDONLY);
		if (fd != -1){
			ioctl (fd,TIOCNOTTY);
			close (fd);
		}
		POPEN pop (cmd.get());
		if (pop.isok()){
			FILE *fout = pop.getfout();
			fprintf (fout,"%s\n",passwd.get());
		}
	}
	return ret;
}

